@@ -1,4 +1,5 @@
 import math
+from re import sub
 from fontTools.pens.recordingPen import RecordingPen, replayRecording
 from fontTools.misc.bezierTools import calcCubicArcLength, splitCubicAtT, calcQuadraticArcLength
 from coldtype.geometry import Rect, Point, Line
@@ -39,8 +40,11 @@ def calcCubicArcLength_cached(a, b, c, d):
 
 class CurveCutter():
     def __init__(self, g, inc=0.0015):
-        #self.pen = g#RecordingPen()
-        self.pen = g.copy()
+        self.pen = RecordingPen()
+        if hasattr(g, "value"):
+            self.pen = g.copy()
+        else:
+            g.replay(self.pen)
         #g.draw(self.pen)
         #self.pen.ensure_fully_closed_path()
         self.inc = inc
@@ -112,11 +116,31 @@ class CurveCutter():
         return out
 
     def subsegmentPoint(self, start=0, end=1):
-        inc = self.inc
         subsegment = self.subsegment(start=start, end=end)
         try:
-            t, (a, b, c) = subsegment[-2]
-            tangent = math.degrees(math.atan2(c[1] - b[1], c[0] - b[0]) + math.pi*.5)
-            return Point(c), tangent
-        except ValueError:
-            return None, None
\ No newline at end of file
+            t, ps = subsegment[-2]
+            if t == "lineTo":
+                a = subsegment[-3][1][0]
+                b, = ps
+                tangent = math.degrees(math.atan2(b[1] - a[1], b[0] - a[0]) + math.pi*.5)
+                return Point(b), tangent
+            elif t == "curveTo":
+                t, (a, b, c) = subsegment[-2]
+                tangent = math.degrees(math.atan2(c[1] - b[1], c[0] - b[0]) + math.pi*.5)
+                return Point(c), tangent
+        except ValueError as e:
+            print(e)
+            return None, None
+
+
+class CurveSample():
+    def __init__(self, idx, pt, e, tan):
+        self.idx = idx
+        self.pt = pt
+        self.e = e
+        self.tan = tan
+    
+    def neighbors(self, prev, next):
+        self.prev = prev
+        self.next = next
+        #self.difft = ((next-prev) + 180) % 360 - 180
\ No newline at end of file
