@@ -179,7 +179,7 @@ class ColdtypeWatchingOperator(bpy.types.Operator):
                 bpy.ops.screen.animation_play() # stop it
 
         for r in candidates:
-            if isinstance(r, b3d_runnable):                
+            if isinstance(r, b3d_runnable):
                 if r.once:
                     if statics:
                         if r.delay:
@@ -381,4 +381,4 @@ def watch(command_file):
     add_3d_panel()
     add_2d_panel()
 
-    print("...waiting for coldtype...")
\ No newline at end of file
+    print("...blender waiting for coldtype...")
\ No newline at end of file
