@@ -1,9 +1,8 @@
 from pathlib import Path
 from defcon import Font as DefconFont
-from coldtype.text.reader import normalize_font_path, StyledString
-from coldtype.pens.datpen import DATPens
+from coldtype.text.reader import normalize_font_path
 from coldtype.interpolation import norm, interp_dict, lerp, loopidx
-from random import Random
+from coldtype.random import random_series
 
 
 def sibling(root, file):
@@ -12,6 +11,39 @@ def sibling(root, file):
 def raw_ufo(path):
     return DefconFont(normalize_font_path(path))
 
+def quick_ufo(path
+    , familyName
+    , styleName="Regular"
+    , versionMajor=1
+    , versionMinor=0
+    , unitsPerEm=1000
+    , descender=-250
+    , ascender=750
+    , capHeight=750
+    , xHeight=500
+    ):
+    np:Path = Path(path).expanduser().resolve()
+    
+    if not np.exists():
+        np.parent.mkdir(exist_ok=True, parents=True)
+        ufo = DefconFont()
+        ufo.save(str(np))
+    
+    ufo = DefconFont(str(np))
+
+    ufo.info.familyName = familyName
+    ufo.info.styleName = styleName
+    ufo.info.versionMajor = versionMajor
+    ufo.info.versionMinor = versionMinor
+    ufo.info.unitsPerEm = unitsPerEm
+    ufo.info.descender = descender
+    ufo.info.xHeight = xHeight
+    ufo.info.capHeight = capHeight
+    ufo.info.ascender = ascender
+
+    return ufo
+
+
 def ßhide(el):
     return None
 
@@ -26,27 +58,6 @@ def cycle_idx(arr, idx):
     else:
         return idx
 
-def random_series(start=0, end=1, seed=0, count=5000):
-    rnd = Random()
-    rnd.seed(seed)
-    rnds = []
-    for x in range(count):
-        rnds.append(start+rnd.random()*(end-start))
-    return rnds
-
-def show_points(pen, style, offcurves=True, filter=lambda i: True):
-    pt_labels = DATPens()
-    if offcurves:
-        def labeller(idx, x, y):
-            if filter(idx):
-                pt_labels.append(StyledString(str(idx), style).pen().translate(x, y))
-        pen.map_points(labeller)
-    else:
-        for idx, (m, pts) in enumerate(pen.value):
-            if len(pts) > 0 and filter(idx):
-                pt_labels += StyledString(str(idx), style).pen().translate(*pts[-1])
-    return pt_labels
-
 _by_uni = None
 _by_glyph = None
 _class_lookup = None
@@ -61,7 +72,7 @@ def _populate_glyphs_unis():
 
     #try:
     if True:
-        lines = (Path(__file__).parent.parent / "assets/glyphNamesToUnicode.txt").read_text().split("\n")
+        lines = (Path(__file__).parent / "assets/glyphNamesToUnicode.txt").read_text().split("\n")
 
         for l in lines:
             if l.startswith("#"):
