@@ -3,14 +3,19 @@ from pathlib import Path
 from runpy import run_path
 from functools import partial
 from tempfile import NamedTemporaryFile
+from subprocess import run
+from typing import Union
+from coldtype.blender import BlenderIO
 
-from coldtype.renderable import renderable
-from coldtype.renderable.animation import animation
+from coldtype.renderable import renderable, ColdtypeCeaseConfigException, runnable, animation, aframe, ui
 
-from coldtype.renderer.utils import Watchable
+from coldtype.renderer.utils import Watchable, on_linux, on_mac, on_windows
 from coldtype.renderer.config import ColdtypeConfig
 from coldtype.helpers import sibling
 from coldtype.text.reader import ALL_FONT_DIRS
+from coldtype.geometry.rect import Rect
+from coldtype.timing import Timeline
+from coldtype.timing.viewer import timeViewer
 
 try:
     from docutils.core import publish_doctree
@@ -18,24 +23,28 @@ except ImportError:
     publish_doctree = None
 
 
-def apply_syntax_mods(source_code, renderer=None):
+def apply_syntax_mods(filepath, source_code, renderer=None):
     codepath_offset = 0
 
     def inline_arg(p):
         path = Path(inline.strip()).expanduser().absolute()
         if renderer:
             if path not in renderer.watchee_paths():
-                renderer.watchees.append([Watchable.Source, path, None])
+                renderer.add_watchee([Watchable.Source, path, None])
         src = path.read_text()
         codepath_offset = len(src.split("\n"))
         return src
     
     def inline_other(x):
         cwd = Path.cwd()
-        path = Path(cwd / (x.group(1).replace(".", "/")+".py"))
+        m = x.group(1)
+        if m.startswith("."):
+            path = filepath.parent / (m[1:] + ".py")
+        else:
+            path = Path(cwd / (m.replace(".", "/")+".py"))
         if renderer:
             if path not in renderer.watchee_paths():
-                renderer.watchees.append([Watchable.Source, path, None])
+                renderer.add_watchee([Watchable.Source, path, None])
         src = path.read_text()
         codepath_offset = len(src.split("\n"))
         return src
@@ -45,31 +54,34 @@ def apply_syntax_mods(source_code, renderer=None):
             source_code = inline_arg(inline) + "\n" + source_code
 
     source_code = re.sub(r"from ([^\s]+) import \* \#INLINE", inline_other, source_code)
-    source_code = re.sub(r"\-\.[A-Za-z_ƒ]+([A-Za-z_0-9]+)?\(", ".nerp(", source_code)
-    source_code = re.sub(r"([\s]+)Ƨ\(", r"\1nerp(", source_code)
+    #source_code = re.sub(r"ℛ", "return ", source_code)
+    #source_code = re.sub(r"\-\.[A-Za-z_ƒ]+([A-Za-z_0-9]+)?\(", ".nerp(", source_code)
+    #source_code = re.sub(r"([\s]+)Ƨ\(", r"\1nerp(", source_code)
     #source_code = re.sub(r"λ[\s]{0,3}\.", "lambda p: p.", source_code)
     #source_code = re.sub(r"λ\s?([/\.\@]{1,2})", r"lambda xxx: xxx\1", source_code)
-    #source_code = re.sub(r"λ\.", "lambda x: x.", source_code)
+    #source_code = re.sub(r"ι,λ\.", "lambda ι, λ__: λ__.", source_code)
+    source_code = re.sub(r"λ(\s+)?\.", "lambda λ__: λ__.", source_code)
+    source_code = re.sub(r"λ__", "λ", source_code)
     #source_code = re.sub(r"λ", "lambda ", source_code)
-    #source_code = re.sub(r"ßDPS\(([^\)]+)\)", r"(ß:=DPS(\1))", source_code)
+    #source_code = re.sub(r"ßDPS\(([^\)]+)\)", r"(ß:=P(\1))", source_code)
 
-    while "nerp(" in source_code:
-        start = source_code.find("nerp(")
-        end = -1
-        i = 5
-        depth = 1
-        c = source_code[start+i]
-        while depth > 0 and c:
-            #print(c, depth)
-            if c == "(":
-                depth += 1
-            elif c== ")":
-                depth -= 1
-            i += 1
-            c = source_code[start+i]
-        end = start+i
-        source_code = source_code[:start] + "noop()" + source_code[end:]
-        #print(start, end)
+    # while "nerp(" in source_code:
+    #     start = source_code.find("nerp(")
+    #     end = -1
+    #     i = 5
+    #     depth = 1
+    #     c = source_code[start+i]
+    #     while depth > 0 and c:
+    #         #print(c, depth)
+    #         if c == "(":
+    #             depth += 1
+    #         elif c== ")":
+    #             depth -= 1
+    #         i += 1
+    #         c = source_code[start+i]
+    #     end = start+i
+    #     source_code = source_code[:start] + "noop()" + source_code[end:]
+    #     #print(start, end)
     
     if renderer:
         renderer._codepath_offset = codepath_offset
@@ -120,18 +132,18 @@ def read_source_to_tempfile(filepath:Path,
         data_out["frontmatter"] = fm
         frag = fragment_fromstring(md, create_parent=True)
         blocks = []
-        for python in frag.findall("./pre/code[@class='python']"):
+        for python in frag.findall("./pre/code[@class='language-python']"):
             blocks.append(python.text)
         source_code = "\n".join(blocks)
         if codepath and codepath.exists():
             codepath.unlink()
         with NamedTemporaryFile("w", prefix="coldtype_md_src", suffix=".py", delete=False) as tf:
-            mod_src, _ = apply_syntax_mods(source_code, renderer)
+            mod_src, _ = apply_syntax_mods(filepath, source_code, renderer)
             tf.write(mod_src)
             codepath = Path(tf.name)
     
     elif filepath.suffix == ".py":
-        source_code, _ = apply_syntax_mods(filepath.read_text(), renderer)
+        source_code, _ = apply_syntax_mods(filepath, filepath.read_text(), renderer)
         if codepath and codepath.exists():
             codepath.unlink()
         with NamedTemporaryFile("w", prefix=f"coldtype__{filepath.stem}_", suffix=".py", delete=False) as tf:
@@ -143,10 +155,13 @@ def read_source_to_tempfile(filepath:Path,
     return codepath, data_out
 
 
-def run_source(filepath, codepath, **kwargs):
+def run_source(filepath, codepath, inputs, memory, **kwargs):
     return run_path(str(codepath), init_globals={
         "__COLDTYPE__": True,
         "__FILE__": filepath,
+        "__inputs__": inputs,
+        "__memory__": memory,
+        "__as_config__": False,
         "__sibling__": partial(sibling, filepath),
         **kwargs})
 
@@ -164,18 +179,24 @@ def find_renderables(
     filepath:Path,
     codepath:Path,
     program:dict,
-    viewer_solos=[],
-    function_filters=[],
-    class_filters=[],
     output_folder_override=None,
+    blender_io=None,
+    args=None,
     ):
     all_rs = []
     filtered_rs = []
     
     for k, v in program.items():
-        if isinstance(v, renderable) and not v.hidden:
+        if (isinstance(v, renderable) or isinstance(v, runnable)) and not v.hidden:
+            if v.cond is not None:
+                if callable(v.cond) and not v.cond():
+                    continue
+                elif not v.cond:
+                    continue
+            
             if v not in all_rs:
                 all_rs.append(v)
+        
         elif k == "RENDERABLES":
             for r in v:
                 all_rs.append(r)
@@ -188,12 +209,26 @@ def find_renderables(
         r.codepath = codepath
         r.output_folder = renderable_to_output_folder(
             filepath, r, override=output_folder_override)
+
+        if hasattr(r, "blender_io"):
+            r.blender_io = blender_io
+
         r.post_read()
 
     if any([r.solo for r in all_rs]):
         filtered_rs = [r for r in all_rs if r.solo]
     else:
         filtered_rs = all_rs
+    
+    return filtered_rs
+
+
+def filter_renderables(filtered_rs,
+    viewer_solos=[],
+    function_filters=[],
+    class_filters=[],
+    previewing=False,
+    ):
         
     if function_filters:
         function_patterns = function_filters
@@ -219,6 +254,13 @@ def find_renderables(
                     print("cf regex compilation error", e)
         filtered_rs = matches
     
+    if previewing:
+        matches = []
+        for r in filtered_rs:
+            if not r.render_only:
+                matches.append(r)
+        filtered_rs = matches
+    
     if len(viewer_solos) > 0:
         viewer_solos = [vs%len(filtered_rs) for vs in viewer_solos]
 
@@ -237,7 +279,9 @@ class SourceReader():
         code:str=None,
         renderer=None,
         runner:str="default",
+        inputs=None,
         cli_args=None,
+        use_blender=False,
         ):
         self.filepath = None
         self.codepath = None
@@ -245,8 +289,11 @@ class SourceReader():
         self.dirpath = None
         self.should_unlink = False
         self.program = None
+        self.candidates = None
         self.renderer = renderer
         self.runner = runner
+        self.inputs = inputs or []
+        self.use_blender = use_blender
 
         self.config = None
         self.read_configs(cli_args, filepath)
@@ -254,43 +301,113 @@ class SourceReader():
         if filepath or code:
             self.reset_filepath(filepath, code)
     
+    @staticmethod
+    def Script(name):
+        root = Path(__file__).parent.parent
+        if name.startswith("script:"):
+            script = root / (name.replace(":", "s/") + ".py")
+            if script.exists():
+                return script
+        return False
+
+    @staticmethod
+    def Demo(name):
+        root = Path(__file__).parent.parent
+        if not name:
+            return root / "demo/demo.py"
+        elif name == "demo":
+            return root / "demo/demo.py"
+        elif name == "blank":
+            return root / "demo/blank.py"
+        elif name == "boiler":
+            return root / "demo/boiler.py"
+        elif name == "glyphloop" or name == "gl":
+            return root / "demo/glyphloop.py"
+        elif name == "glyphs" or name == "g":
+            return root / "demo/glyphs.py"
+        elif name == "midi":
+            return root / "demo/midi.py"
+        elif name == "vf":
+            return root / "demo/vf.py"
+        elif name == "viewseq":
+            return root / "demo/viewseq.py"
+        elif name == "docstrings":
+            return root / "demo/docstrings.py"
+        return name
+    
+    @staticmethod
+    def LoadDemo(demoname, **inputs):
+        return SourceReader(
+            SourceReader.Demo(demoname),
+            inputs=inputs).renderables()
+    
+    @staticmethod
+    def FrameResult(name, frame, inputs={}, renderer_state=None):
+        filepath = SourceReader.Demo(name)
+        sr = SourceReader(filepath, inputs=inputs)
+        sr.unlink()
+        return sr.frame_results(frame,
+            renderer_state=renderer_state)
+    
     def read_configs(self, args, filepath):
+        embedded = Path(__file__).parent / ".coldtype.py"
         proj = Path(".coldtype.py")
         user = Path("~/.coldtype.py").expanduser()
-        files = [user, proj]
+        
+        if on_windows():
+            os = Path(".coldtype.win.py")
+        elif on_mac():
+            os = Path(".coldtype.mac.py")
+        elif on_linux():
+            os = Path(".coldtype.lin.py")
+
+        files = [embedded, user, proj, os]
+
         if args and hasattr(args, "config") and args.config:
             if args.config == "0":
                 files = []
             else:
-                if args.config == ".":
-                    args.config = args.file
-                files.append(Path(args.config).expanduser())
+                pass
+                #if args.config == ".":
+                #    args.config = args.file
+                #files.append(Path(args.config).expanduser())
 
         if filepath:
-            files.append(Path(filepath).expanduser())
+            fp = Path(filepath).expanduser()
+            if fp.exists() and not fp.is_dir():
+                if "# .coldtype" in fp.read_text():
+                    files.append(fp)
         
-        if args and args.file:
+        if args and args.file and args.config != "0":
             fp = Path(args.file).expanduser()
             if fp.exists() and not fp.is_dir():
-                files.append(fp)
+                if "# .coldtype" in fp.read_text():
+                    files.append(fp)
 
         py_config = {}
         for p in files:
-            if p.exists():
+            if p.exists() and p.suffix == ".py":
                 try:
                     py_config = run_path(str(p), init_globals={
                         "__FILE__": p,
+                        "__inputs__": self.inputs,
+                        "__memory__": {},
+                        "__as_config__": True,
                         "__sibling__": partial(sibling, p),
                     })
-                    #self.midi_mapping = py_config.get("MIDI", self.midi_mapping)
-                    #self.hotkey_mapping = py_config.get("HOTKEYS", self.hotkey_mapping)
+                    
                     self.config = ColdtypeConfig(py_config, self.config if self.config else None, args)
 
                     for f in self.config.font_dirs:
-                        ALL_FONT_DIRS.insert(0, f)
+                        if f not in ALL_FONT_DIRS:
+                            ALL_FONT_DIRS.insert(0, f)
                 except Exception as e:
-                    print("Failed to load config", p)
-                    print("Exception:", e)
+                    if isinstance(e, ColdtypeCeaseConfigException):
+                        pass
+                    else:
+                        print("Failed to load config", p)
+                        print("Exception:", e)
+                        pass
         
         if len(files) == 0 or not self.config:
             self.config = ColdtypeConfig({}, None, args)
@@ -298,18 +415,35 @@ class SourceReader():
         self.config.args = args
         #print(self.config.values())
     
-    def find_sources(self, dirpath):
-        sources = list(dirpath.glob("*.py"))
-        sources.extend(list(dirpath.glob("*.md")))
+    def find_sources(self, dirpath, recursive=False):
+        prefix = "**/" if recursive else ""
+        globber = f"{prefix}*.py"
+        if self.filepath:
+            globber = f"{prefix}*" + self.filepath.suffix
+        sources = list(dirpath.glob(globber))
+        #sources.extend(list(dirpath.glob("*.md")))
 
         valid_sources = []
         for p in sources:
             if not p.name.startswith("_"):
                 valid_sources.append(p)
-            valid_sources = sorted(valid_sources, key=lambda p: p.stem)
-        
+            valid_sources = sorted(valid_sources, key=lambda p: str(p.relative_to(dirpath)))
+            valid_sources = sorted(valid_sources, key=lambda p: str(p.relative_to(dirpath)).count("/") > 0)
+
+        #for p in valid_sources:
+        #    print(p.relative_to(dirpath))
+
         return valid_sources
     
+    def blender_io(self):
+        #if not self.use_blender and not self.config.blender_watch:
+        #    return None
+
+        bf = self.config.blender_file
+        if not bf:
+            bf = self.filepath.parent / "blends" / (self.filepath.stem + ".blend")
+        return BlenderIO(bf)
+    
     def normalize_filepath(self, filepath:Path, dirindex=0):
         if isinstance(filepath, str):
             filepath = Path(filepath)
@@ -321,7 +455,8 @@ class SourceReader():
             #filepath = sorted(list(filepath.glob("*.py")), key=lambda p: p.stem)[dirindex]
             filepath = self.find_sources(self.dirpath)[dirindex]
         else:
-            self.dirpath = filepath.parent
+            if self.dirpath is None:
+                self.dirpath = filepath.parent
 
         if not filepath.exists():
             with_py = (filepath.parent / (filepath.stem + ".py"))
@@ -332,6 +467,10 @@ class SourceReader():
         
         return filepath
     
+    def adjacents(self):
+        if self.dirpath:
+            return self.find_sources(self.dirpath)
+    
     def reset_filepath(self, filepath:Path, code:str=None, reload:bool=True, dirdirection=0):
         self.unlink()
         self.should_unlink = False
@@ -343,7 +482,7 @@ class SourceReader():
             #pys = sorted(list(self.dirpath.glob("*.py")), key=lambda p: p.stem)
             curr = pys.index(self.filepath)
             adj = (curr + dirdirection) % len(pys)
-            print(curr, adj)
+            #print(curr, adj)
             filepath = pys[adj]
         
         if filepath:
@@ -361,15 +500,48 @@ class SourceReader():
         
         return self.filepath
     
-    def reload(self, code:str=None):
+    def reload(self,
+        code:str=None,
+        output_folder_override=None
+        ):
         if code:
             self.write_code_to_tmpfile(code)
         
         self.codepath, self.data_out = read_source_to_tempfile(self.filepath, self.codepath, renderer=self.renderer)
+        
+        memory = {}
+        if self.renderer:
+            memory = self.renderer.state.memory
+        
+        source_code = self.codepath.read_text()
+        version = None
+
+        if re.findall(r"VERSIONS\s?=", source_code):
+            versions = re.findall(r"VERSIONS\s?=.*\#\/VERSIONS", source_code, re.DOTALL)[0]
+            versions = eval(re.sub("VERSIONS\s?=", "", versions))
+            if not isinstance(versions, dict):
+                versions = {str(idx):v for idx, v in enumerate(versions)}
+            versions = {k:{**v, **dict(key=k)} for k,v in versions.items()}
+            versions = list(versions.values())
+            vi = self.renderer.source_reader.config.version_index
+            version = versions[vi]
+            self.renderer.state.versions = versions
+
+            source_code = source_code.replace("ƒVERSION", version["key"])
+            self.codepath.write_text(source_code)
+
+
         self.program = run_source(
             self.filepath,
             self.codepath,
-            __RUNNER__=self.runner)
+            self.inputs,
+            memory,
+            __RUNNER__=self.runner,
+            __BLENDER__=self.blender_io(),
+            __VERSION__=version)
+        
+        self.candidates = self.renderable_candidates(
+            output_folder_override, self.config.add_time_viewers)
     
     def write_code_to_tmpfile(self, code):
         if self.filepath:
@@ -381,30 +553,60 @@ class SourceReader():
         self.filepath = Path(tf.name)
         self.should_unlink = True
     
+    def renderable_candidates(self,
+        output_folder_override=None,
+        add_time_viewers=False,
+        ):
+        candidates = find_renderables(
+            self.filepath,
+            self.codepath,
+            self.program,
+            output_folder_override,
+            blender_io=self.blender_io(),
+            args=self.renderer.args if self.renderer else None)
+        
+        if len(candidates) == 0:
+            candidates.append(Programs.Blank())
+        
+        if add_time_viewers:
+            out = []
+            for c in candidates:
+                if (isinstance(c, animation) and
+                    not isinstance(c, aframe) and
+                    not isinstance(c, ui)
+                    ):
+                    out.extend(timeViewer(c))
+                out.append(c)
+            return out
+        else:
+            return candidates
+    
     def renderables(self,
         viewer_solos=[],
         function_filters=[],
         class_filters=[],
-        output_folder_override=None,
+        previewing=False,
         ):
         if not function_filters and self.config.function_filters:
             function_filters = self.config.function_filters
-        return find_renderables(
-            self.filepath,
-            self.codepath,
-            self.program,
-            viewer_solos,
-            function_filters,
-            class_filters,
-            output_folder_override)
+        
+        return filter_renderables(self.candidates,
+            viewer_solos=viewer_solos,
+            function_filters=function_filters,
+            class_filters=class_filters,
+            previewing=previewing)
     
-    def frame_results(self, frame, class_filters=[]):
+    def frame_results(self, frame, class_filters=[], renderer_state=None):
         rs = self.renderables(class_filters=class_filters)
         res = []
         for r in rs:
-            ps = r.passes(None, None, indices=[frame])
+            if isinstance(r, runnable):
+                res.append([r, None])
+                continue
+            
+            ps = r.passes(None, renderer_state, indices=[frame])
             for p in ps:
-                res.append([r, r.run_normal(p)])
+                res.append([r, r.run_normal(p, renderer_state=renderer_state)])
         return res
     
     def unlink(self):
@@ -414,4 +616,53 @@ class SourceReader():
         if self.codepath:
             if self.codepath.exists():
                 self.codepath.unlink()
-        return self
\ No newline at end of file
+        return self
+
+
+class Programs():
+    @staticmethod
+    def Demo():
+        return SourceReader.LoadDemo("demo")[0]
+    
+    @staticmethod
+    def Blank():
+        return SourceReader.LoadDemo("blank")[0]
+    
+    @staticmethod
+    def Glyphs(
+        font=None,
+        fontSize=54,
+        showChars=False,
+        rect=(1080, 1080)
+        ):
+        return SourceReader.LoadDemo("glyphs", **locals())[0]
+
+    @staticmethod
+    def Midi(
+        file=None,
+        duration=None,
+        bpm=None,
+        lookup=None,
+        text=True,
+        fps=30,
+        rect=(1080, 540),
+        log=False,
+        preview_only=True,
+        ):
+        return SourceReader.LoadDemo("midi", **locals())[0]
+    
+    @staticmethod
+    def VF(
+        font=None,
+        text="A",
+        font_size=None,
+        positions=(0, 1),
+        stroke=False,
+        seed=0,
+        shuffle=False,
+        animate=False,
+        rect=(1080, 1080),
+        log=False,
+        preview_only=True,
+        ):
+        return SourceReader.LoadDemo("vf", **locals())[0]
\ No newline at end of file
