@@ -1,12 +1,12 @@
-#from coldtype.pens.draftingpens import DraftingPen, DraftingPens
-from coldtype.pens.draftingpens import DraftingPens
+from coldtype.runon.path import P
 from coldtype.geometry import Rect, Point
 
 from coldtype.text.shaper import segment
-from coldtype.text.reader import Style, StyledString, FittableMixin, Font, normalize_font_path, SegmentedString,  _PenClass, _PensClass
+from coldtype.text.reader import Style, StyledString, FittableMixin, Font, SegmentedString
 
-import inspect
 from collections import namedtuple
+from typing import Callable
+
 
 class GrafStyle():
     def __init__(self, leading=10, x="centerx", xp=0, width=0, **kwargs):
@@ -17,9 +17,9 @@ class GrafStyle():
 
 
 class Graf():
-    def __init__(self, lines, container, style=None, **kwargs):
+    def __init__(self, lines, container, style=None, no_frames=False, **kwargs):
         if isinstance(container, Rect):
-            self.container = _PenClass().rect(container)
+            self.container = P().rect(container)
         else:
             self.container = container
         if style and isinstance(style, GrafStyle):
@@ -28,6 +28,8 @@ class Graf():
             self.style = GrafStyle(leading=style)
         else:
             self.style = GrafStyle(**kwargs)
+        
+        self.no_frames = no_frames
         self.lines = lines
     
     def lineRects(self):
@@ -63,21 +65,16 @@ class Graf():
             l.fit(fw)
         return self
     
-    def pens(self, align=False):
+    def pens(self):
         rects = self.lineRects()
-        pens = _PensClass()
+        pens = P()
         for idx, l in enumerate(self.lines):
             r = rects[idx]
             dps = l.pens().translate(r.x, r.y) # r.x
-            dps.addFrame(Rect(r.x, r.y, r.w, r.h))
-            #if align:
-            #    dps.container = Rect(0, r.y, r.w, r.h)
-            #    dps.align(dps.container, x=self.style.x, y=None)
+            if not self.no_frames:
+                dps.data(frame=Rect(r.x, r.y, r.w, r.h))
             pens.append(dps)
         return pens
-    
-    def fpa(self, rect=None):
-        return self.fit().pens().align(rect or self.container.ambit())
 
 
 class Lockup(FittableMixin):
@@ -115,18 +112,18 @@ class Lockup(FittableMixin):
                 if self.nestSlugs:
                     pens.append(dps)
                 else:
-                    pens.extend(dps._pens)
+                    pens.extend(dps._els)
             else:
                 dps = s.pen()
                 dps.translate(x_off, 0)
                 pens.append(dps)
-            x_off += dps.ambit(th=0).w
+            x_off += dps.ambit(tx=0).w
             try:
                 x_off += s.margin[1]
                 x_off += s.strings[-1].tracking
             except:
                 pass
-        return _PensClass(pens)
+        return P(pens)
     
     def pen(self):
         return self.pens().pen()
@@ -185,7 +182,7 @@ class Composer():
         """
         Structured representation of the multi-line text
         
-        In the return ``DATPens``, each line will be a ``DATPens``, then within those lines, each glyph/ligature for that line will be an individual ``DATPen``
+        In the result, each line will be a `P`, then within those lines, each glyph/ligature for that line will be an individual `P`
         """
         return self.graf.pens()
     
@@ -200,11 +197,48 @@ def StSt(text,
     font,
     font_size=24,
     rect=Rect(1080, 1080),
-    xa="mdx",
-    **kwargs):
+    strip=True,
+    multiline=False,
+    #xa="mdx",
+    **kwargs) -> P:
+    """Set a line of text with a single Style object,
+    passed either with it’s constituent parts (i.e. kwargs)
+    or as an actual `Style` object.
+
+    #### Examples:
     
+    ```python
+    @renderable()
+    def stst_1(r):
+        # here the styling arguments are "flat"
+        return (StSt("COLDTYPE", Font.ColdObvi(), 100, wdth=0)
+            .align(r))
+    ```
+    
+    ```python
+    @renderable()
+    def stst_2(r):
+        # here the styling arguments are encapsulated in the Style object
+        return (StSt("COLDTYPE", Style(Font.ColdObvi(), 100, wdth=1))
+            .align(r))
+    ```
+    """
+
+    if not isinstance(text, str):
+        text = "\n".join(text)
+    else:
+        if strip:
+            text = text.strip()
+    
+    styles = []
+
     if isinstance(font, Style):
         style = font
+    elif isinstance(font, dict):
+        style = Style(**{**font, **kwargs})
+    elif isinstance(font, list) and isinstance(font[0], Style):
+       style = font[0]
+       styles = font
     else:
         style = Style(font, font_size, **kwargs)
     
@@ -212,21 +246,59 @@ def StSt(text,
     leading = kwargs.get("leading", 10)
 
     if "\n" in text:
-        lockup = Composer(rect, text, style, fit=fit, leading=leading).pens()
-        if xa:
-            lockup = lockup.xa(xa)
-        for l in lockup:
-            l._frame = None
+        lines = P()
+        for idx, l in enumerate(text.split("\n")):
+            if styles:
+                _style = styles[idx]
+            else:
+                _style = style
+            lines.append(StSt(l, _style, rect=rect, strip=strip, **{**kwargs, **dict(multline=False)}))
+        return lines.stack(leading)
     else:
-        lockup = StyledString(text, style)
-        if fit:
-            lockup.fit(fit)
-        lockup = lockup.pens()
-    return lockup
+        if style.fallback:
+            lockup = Slug(text, style, style.fallback)
+            if fit:
+                lockup.fit(fit)
+            lockup = lockup.pens(flat=False)
+        else:
+            lockup = StyledString(text, style)
+            if fit:
+                lockup.fit(fit)
+            lockup = lockup.pens()
+        
+        if multiline:
+            return P([lockup])
 
-GlyphwiseGlyph = namedtuple("GlyphwiseGlyph", ["i", "c", "e"])
+        #lockup._stst_style = style
+        return lockup
 
-def Glyphwise(st, styler):
+
+GlyphwiseGlyph = namedtuple("GlyphwiseGlyph", ["i", "c", "e", "l", "li"])
+
+
+def Glyphwise(st:str
+    , styler:Callable[[GlyphwiseGlyph], Style]
+    , start:int=0
+    , line:int=0
+    , multiline=False) -> P:
+    """
+    Build text by applying unique style to each glyph.
+
+    Style is determined by a `styler` function (usually a lambda)
+    that is given a `GlyphwiseGlyph` containing information about
+    the glyph and its position (index, etc.); styler function
+    must return a Style object to be used for styling
+
+    #### Examples:
+    
+    ```python
+    @renderable()
+    def glyphwise(r):
+        return (Glyphwise("COLDTYPE", lambda x:
+            Style(Font.ColdObvi(), 200, wdth=x.e))
+            .align(r))
+    ```
+    """
     # TODO possible to have an implementation
     # aware of a non-1-to-1 mapping of characters
     # to glyphs? seems very difficult if not impossible,
@@ -242,14 +314,49 @@ def Glyphwise(st, styler):
     #print(glyphs)
     #print([g.name for g in glyphs])
 
+    def except_reverse():
+        raise Exception("r=1 not possible in a Glyphwise; please use a .reversePens() after the Glyphwise constructor")
+
+    def run_styler(g):
+        styles = styler(g)
+        if isinstance(styles, Style):
+            if styles.reverse:
+                except_reverse()
+            return styles, None
+        else:
+            if isinstance(styles[1], dict):
+                if styles[0].reverse:
+                    except_reverse()
+                return styles[0], styles[0].mod(**styles[1])
+            else:
+                if styles[0].reverse:
+                    except_reverse()
+                return styles
+
+    if len(st) == 1:
+        return StSt(st,
+            run_styler(
+                GlyphwiseGlyph(0, st, 0, line, 0))[0], strip=False)
+
+    try:
+        lines = st.split("\n")
+        if len(lines) > 1 or multiline:
+            gs = []
+            start = 0
+            for lidx, l in enumerate(lines):
+                gs.append(Glyphwise(l, styler, start=start, line=lidx))
+                start += len(l)
+            return P(gs).stack()
+    except AttributeError:
+        pass
+
     def kx(dps, idx):
         return dps[idx].ambit().x
     
     def krn(off, on, idx):
         return kx(off, idx) - kx(on, idx)
 
-    arg_count = len(inspect.signature(styler).parameters)
-    dps = DraftingPens()
+    dps = P()
     prev = 0
     tracks = []
 
@@ -269,33 +376,51 @@ def Glyphwise(st, styler):
             test = [test]
         
         e = idx / (len(st)-1)
-        gg = GlyphwiseGlyph(idx, c, e)
+        gg = GlyphwiseGlyph(idx+start, c, e, line, idx)
 
-        if arg_count == 1:
-            skon = styler(gg)
-        else:
-            skon = styler(idx, gg)
-        
-        skoff = skon.mod(kern=0)
+        skon, skon_tweak = run_styler(gg)
+        skoff = skon.mod(kern=0, kern_pairs={}, kp={}, tu=0)
 
         #test_list = [t for sublist in test for t in sublist]
         #print(test_list)
         test_str = "".join([t if isinstance(t, str) else "".join(t) for t in test])
         #print(c, test_str, target, test, tcount)
 
-        tkon = StSt(test_str, skon)
-        tkoff = StSt(test_str, skoff)
+        tkon = StSt(test_str, skon.mod(no_shapes=True), strip=False)
+        if skon_tweak is None:
+            tkoff = StSt(test_str, skoff, strip=False)
+            tkoff_tweak = None
+        else:
+            tkoff = StSt(test_str, skoff.mod(no_shapes=True), strip=False)
+            skoff_tweak = skon_tweak.mod(kern=0, kern_pairs={}, kp={}, tu=0)
+            tkoff_tweak = StSt(test_str, skoff_tweak, strip=False)
 
         if idx == 0:
-            dps.append(tkoff[0])
+            if tkoff_tweak:
+                tkoff_frame = tkoff[0].data("frame")
+                tx = skon_tweak.input["kwargs"].get("tx", 0)
+                ty = skon_tweak.input["kwargs"].get("ty", 0)
+                tkoff_glyph = tkoff_tweak[0].align(tkoff_frame, tx=tx, ty=ty)
+                tkoff_glyph.data(frame=tkoff_frame)
+            else:
+                tkoff_glyph = tkoff[0]#.copy(with_data=True)
+            
+            dps.append(tkoff_glyph)
             prev = krn(tkoff, tkon, 1)
         if target > 0:
             _prev = krn(tkoff, tkon, 1)
             prev_av = (prev+_prev)/2
-            #print(prev_av, kx(tkoff, 1))
 
-            dps.append(tkoff[1].copy(with_data=True)
-                .translate(-kx(tkoff, 1), 0))
+            if tkoff_tweak:
+                tkoff_frame = tkoff[1].data("frame")
+                tx = skon_tweak.input["kwargs"].get("tx", 0)
+                ty = skon_tweak.input["kwargs"].get("ty", 0)
+                tkoff_glyph = tkoff_tweak[1].align(tkoff_frame, tx=tx, ty=ty)
+                tkoff_glyph.data(frame=tkoff_frame)
+            else:
+                tkoff_glyph = tkoff[1].copy(with_data=True)
+            
+            dps.append(tkoff_glyph.translate(-kx(tkoff, 1), 0))
             tracks.append(-prev_av)
 
             if tcount > 2:
