@@ -1,39 +1,27 @@
-from pathlib import Path
 from collections import OrderedDict
-from functools import partial, lru_cache
+from functools import partial
 
-import unicodedata, math, os, re
+import unicodedata, math
 
 from fontTools.misc.transform import Transform
 from fontTools.pens.transformPen import TransformPen
-from fontTools.pens.svgPathPen import SVGPathPen
-from fontTools.pens.recordingPen import RecordingPen, replayRecording
-from fontTools.pens.boundsPen import ControlBoundsPen, BoundsPen
-from fontTools.ttLib.ttFont import TTFont
 
-from coldtype.color import normalize_color
-from coldtype.pens.draftingpen import DraftingPen
-from coldtype.pens.draftingpens import DraftingPens
-from coldtype.pens.datpen import DATPen, DATPens
-from coldtype.geometry import Rect, Point
+from coldtype.color import normalize_color, rgb
+from coldtype.runon.path import P
+from coldtype.geometry import Rect
+from coldtype.text.font import Font, normalize_font_path, normalize_font_prefix, FontNotFoundException, ALL_FONT_DIRS, _prefixes
 
-from typing import Optional, Callable, Union
+from typing import Union
 
-_PenClass = DATPen
-_PensClass = DATPens
-
-# try:
-#     from fontgoggles.font import getOpener
-#     from fontgoggles.font.baseFont import BaseFont
-#     from fontgoggles.font.otfFont import OTFFont
-#     from fontgoggles.misc.textInfo import TextInfo
-# except ModuleNotFoundError:
-
-from coldtype.fontgoggles.font import getOpener
-from coldtype.fontgoggles.font.baseFont import BaseFont
-from coldtype.fontgoggles.font.otfFont import OTFFont
 from coldtype.fontgoggles.misc.textInfo import TextInfo
 
+try:
+    from blackrenderer.font import BlackRendererFont
+    #from blackrenderer.backends.pathCollector import PathCollectorSurface, PathCollectorRecordingPen
+    from coldtype.text.colr.brsurface import BRPathCollectorSurface, BRPathCollectorRecordingPen
+except ImportError:
+    BlackRendererFont = None
+    pass
 
 class FittableMixin():
     def textContent(self):
@@ -60,217 +48,129 @@ class FittableMixin():
         return self
 
 
-_prefixes = [
-    ["¬", "~/Library/Fonts"],
-    ["", "/Library/Fonts"]
-]
-
-# TODO windows & linux?
-ALL_FONT_DIRS = [
-    "/System/Library/Fonts",
-    "/Library/Fonts",
-    "~/Library/Fonts",
-]
-
-class FontNotFoundException(Exception):
-    pass
-
-def normalize_font_prefix(path_string):
-    for prefix, expansion in _prefixes:
-        path_string = path_string.replace(prefix, expansion)
-    return Path(path_string).expanduser().resolve()
-
-def normalize_font_path(font, nonexist_ok=False):
-    global _prefixes
-    literal = normalize_font_prefix(str(font))
-    ufo = literal.suffix == ".ufo"
-    if nonexist_ok:
-        return str(literal)
-    if literal.exists() and (not literal.is_dir() or ufo):
-        return str(literal)
-    else:
-        raise FontNotFoundException(literal)
-
-FontCache = {}
-
-class Font():
-    # TODO support glyphs?
-    def __init__(self, path, number=0, cacheable=False):
-        self.path = Path(normalize_font_path(path))
-        numFonts, opener, getSortInfo = getOpener(self.path)
-        self.font:BaseFont = opener(self.path, number)
-        self.font.cocoa = False
-        self.cacheable = cacheable
-        self._loaded = False
-        self.load()
-    
-    def load(self):
-        if self._loaded:
-            return self
-        else:
-            self.font.load(None)
-            self._loaded = True
-            return self
-    
-    @staticmethod
-    def Cacheable(path):
-        if path not in FontCache:
-            FontCache[path] = Font(path, cacheable=True).load()
-        return FontCache[path]
-
-    @lru_cache()
-    def List(regex, regex_dir=None):
-        results = []
-        for dir in ALL_FONT_DIRS:
-            dir = normalize_font_prefix(dir)
-            if regex_dir:
-                if not re.search(regex_dir, str(dir)):
-                    continue
-            for root, dirs, files in os.walk(dir):
-                for dir in dirs:
-                    path = Path(root + "/" + dir)
-                    if path.suffix == ".ufo":
-                        if re.search(regex, dir):
-                            results.append(path)
-                for file in files:
-                    if re.search(regex, file):
-                        path = Path(root + "/" + file)
-                        if path.suffix in [".ttf", ".otf", ".ttc"]:
-                            results.append(path)
-        return results
-
-    def Find(regex, regex_dir=None, index=0):
-        found = Font.List(regex, regex_dir)
-        try:
-            return Font.Cacheable(found[index])
-        except:
-            raise FontNotFoundException()
-    
-    @staticmethod
-    def ColdtypeObviously():
-        return Font.Cacheable(Path(__file__).parent.parent / "demo/ColdtypeObviously-VF.ttf")
-
-    @staticmethod
-    def MutatorSans():
-        return Font.Cacheable(Path(__file__).parent.parent / "demo/MutatorSans.ttf")
-    
-    @staticmethod
-    def RecursiveMono():
-        return Font.Cacheable(Path(__file__).parent.parent / "demo/RecMono-CasualItalic.ttf")
-
 class Style():
     """
     Class for configuring font properties
 
-    **Keyword arguments**
+    #### Keyword arguments
 
-    * ``font``: can either be a ``coldtype.text.Font`` object, a ``pathlib.Path``, or a plain string path
-    * ``fontSize``: standard point-based font-size, expressed as integer
-    * ``tracking`` (aka ``tu``): set the tracking, by default **in font-source-point-size** aka as if the font-size was always 1000; this means tracking is by default done relatively rather than absolutely (aka the relative tracking will not change when you change the fontSize)
-    * ``trackingMode``: set to 0 to set tracking in a classic fontSize-based (defaults to 1, as described just above)
-    * ``space``: set this to override the width of the standard space character (useful when setting text on a curve and the space is getting collapsed)
-    * ``baselineShift`` (aka ``bs``): if an integer, shifts glyphs by that amount in y axis; if a list, shifts glyphs at corresponding index in list by that amount in y axis
-    * ``xShift`` (aka ``xs``): if an integer, shifts glyphs by that amount in x axis; if a list, shifts glyphs at corresponding index in list by that amount in x axis
-    * ``rotate``: rotate glyphs by degree
-    * ``reverse`` (aka ``r``): reverse the order of the glyphs, so that the left-most glyph is first in when vectorized via ``.pens()``
-    * ``removeOverlaps`` (aka ``ro``): automatically use skia-pathops to remove overlaps from the glyphs (useful when using variable ttf fonts)
-    * ``lang``: set language directly, to access language-specific alternate characters/rules
+    * `font`: can either be a `coldtype.text.Font` object, a `pathlib.Path`, or a plain string path
+    * `font_size`: standard point-based font-size, expressed as integer
+    * `tracking` (aka `tu`): set the tracking, by default **in font-source-point-size** aka as if the font-size was always 1000; this means tracking is by default done relatively rather than absolutely (aka the relative tracking will not change when you change the font_size)
+    * `trackingMode`: set to 0 to set tracking in a classic font_size-based (defaults to 1, as described just above)
+    * `space`: set this to override the width of the standard space character (useful when setting text on a curve and the space is getting collapsed)
+    * `baselineShift` (aka `bs`): if an integer, shifts glyphs by that amount in y axis; if a list, shifts glyphs at corresponding index in list by that amount in y axis
+    * `xShift` (aka `xs`): if an integer, shifts glyphs by that amount in x axis; if a list, shifts glyphs at corresponding index in list by that amount in x axis
+    * `rotate`: rotate glyphs by degree
+    * `reverse` (aka `r`): reverse the order of the glyphs, so that the left-most glyph is first in when vectorized via `.pens()`
+    * `removeOverlaps` (aka `ro`): automatically use skia-pathops to remove overlaps from the glyphs (useful when using variable ttf fonts)
+    * `lang`: set language directly, to access language-specific alternate characters/rules
 
-    **Shorthand kwargs**
+    #### Shorthand kwargs
 
-    * ``kp`` for ``kern_pairs`` — a dict of glyphName->[left,right] values in font-space
-    * ``tl`` for ``trackingLimit``
-    * ``bs`` for ``baselineShift``
-    * ``ch`` for ``capHeight`` — a number in font-space; not specified, read from font; specified as 'x', capHeight is set to xHeight as read from font
+    * `kp` for `kern_pairs` — a dict of glyphName->[left,right] values in font-space
+    * `tl` for `trackingLimit`
+    * `bs` for `baselineShift`
+    * `ch` for `capHeight` — a number in font-space; not specified, read from font; specified as 'x', capHeight is set to xHeight as read from font
     """
     def RegisterShorthandPrefix(prefix, expansion):
         global _prefixes
         _prefixes.append([prefix, str(expansion)])
 
     def __init__(self,
-            font:Union[Font, str]=None,
-            fontSize:int=12,            
-            tracking=0,
-            trackingMode=1,
-            kern_pairs=dict(),
-            space=None,
-            baselineShift=0,
-            xShift=None,
-            rotate=0,
-            reverse=False,
-            removeOverlap=False,
-            q2c=False,
-            lang=None,
-            narrower=None,
-            include_blanks=False,
-            palette=0,
-            capHeight=None,
-            data={},
-            layer=None,
-            liga=True,
-            kern=True,
-            fill=(0, 0.5, 1),
-            stroke=None,
-            strokeWidth=0,
-            variations=dict(),
-            variationLimits=dict(),
-            trackingLimit=0,
-            scaleVariations=True,
-            mods=None,
-            features=dict(),
-            increments=dict(),
-            varyFontSize=False,
-            preventHwid=False,
-            fitHeight=None,
-            load_font=True, # should we attempt to load the font?
-            tag=None, # way to differentiate in __eq__
-            **kwargs):
+        font:Union[Font, str]=None,
+        font_size:int=12,
+        tracking=0,
+        trackingMode=1,
+        kern_pairs=dict(),
+        space=None,
+        baselineShift=0,
+        xShift=None,
+        rotate=0,
+        reverse=False,
+        removeOverlap=False,
+        q2c=False,
+        lang=None,
+        narrower=None,
+        fallback=None,
+        palette=0,
+        capHeight=None,
+        ascender=None,
+        descender=None,
+        metrics="c",
+        data={},
+        layer=None,
+        liga=True,
+        kern=True,
+        fill=rgb(0, 0.5, 1),
+        stroke=None,
+        strokeWidth=0,
+        variations=dict(),
+        variationLimits=dict(),
+        trackingLimit=0,
+        scaleVariations=True,
+        rollVariations=False,
+        mods=None,
+        features=dict(),
+        increments=dict(),
+        varyFontSize=False,
+        preventHwid=False,
+        fitHeight=None,
+        meta=dict(),
+        no_shapes=False,
+        show_frames=False,
+        load_font=True, # should we attempt to load the font?
+        tag=None, # way to differentiate in __eq__
+        _stst=False,
+        **kwargs
+        ):
 
         self.input = locals()
         self.input["self"] = None
 
         if load_font:
-            if isinstance(font, Path):
-                font = str(font)
-            if isinstance(font, str):
-                self.font:Font = Font(font)
-                self.font.load() #font._syncLoad(None)
-            else:
-                self.font:Font = font
+            self.font = Font.Normalize(font)
         else:
             self.font = font
 
+        self.meta = meta
+
+        self.fallback = fallback
         self.narrower = narrower
         self.layer = layer
         self.reverse = kwargs.get("r", reverse)
         self.removeOverlap = kwargs.get("ro", removeOverlap)
         self.q2c = q2c
         self.rotate = rotate
-        self.include_blanks = include_blanks
-        self.scaleVariations = scaleVariations
+        self.scaleVariations = kwargs.get("sv", scaleVariations)
+        self.rollVariations = kwargs.get("rv", rollVariations)
         self.tag = tag
+        self._stst = _stst
+        
+        self.metrics = metrics
+        self.capHeight = kwargs.get("ch", capHeight)
+        self.descender = kwargs.get("dsc", descender)
+        self.ascender = kwargs.get("asc", ascender)
 
-        try:        
-            if "OS/2" in self.font.font.ttFont:
-                os2 = self.font.font.ttFont["OS/2"]
-                self.capHeight = os2.sCapHeight if hasattr(os2, "sCapHeight") else 0
-                if self.capHeight == 0:
-                    self.capHeight = os2.sTypoAscender if hasattr(os2, "sTypoAscender") else 750
-            elif hasattr(self.font.font, "info"):
-                self.capHeight = self.font.font.info.capHeight
-            elif hasattr(self.font.font, "defaultInfo"):
-                self.capHeight = self.font.font.defaultInfo.capHeight
+        self.no_shapes = no_shapes
+        self.show_frames = show_frames
 
-            if capHeight: # override whatever the font says
-                if capHeight != "x":
-                    self.capHeight = capHeight
-        except AttributeError:
-            pass
+        self.complete_metrics()
+        if "c" in self.metrics:
+            self._asc = self.capHeight
+        else:
+            self._asc = self.ascender
+        
+        # legacy for older code
+        if kwargs.get("fontSize"):
+            font_size = kwargs.get("fontSize")
 
         if fitHeight:
-            self.fontSize = (fitHeight/self.capHeight)*1000
+            self.fontSize = (fitHeight/self._asc)*1000
         else:
-            self.fontSize = fontSize
+            self.fontSize = font_size
+        
+        self.fontSize = max(self.fontSize, 0)
 
         self.tracking = kwargs.get("t", tracking)
         self.kern_pairs = kwargs.get("kp", kern_pairs)
@@ -306,6 +206,7 @@ class Style():
 
         self.fill = normalize_color(fill)
         self.stroke = normalize_color(stroke)
+        
         if stroke and strokeWidth == 0:
             self.strokeWidth = 1
         else:
@@ -316,20 +217,27 @@ class Style():
         self.variations = dict()
         self.variationLimits = dict()
         self.varyFontSize = varyFontSize
-        
-        if load_font and self.font.font.ttFont:
+
+        if not load_font:
+            return
+        else:
             try:
                 fvar = self.font.font.ttFont['fvar']
             except:
                 fvar = None
             if fvar:
-                for axis in fvar.axes:
+                for axidx, axis in enumerate(sorted(fvar.axes, key=lambda ax: ax.axisTag)):
+                    generic = f"fvar_{axidx}"
                     self.axes[axis.axisTag] = axis
                     self.variations[axis.axisTag] = axis.defaultValue
                     if axis.axisTag == "wdth": # the only reasonable default
                         self.variationLimits[axis.axisTag] = axis.minValue
                     if axis.axisTag in kwargs and axis.axisTag not in variations:
-                        unnormalized_variations[axis.axisTag] = kwargs[axis.axisTag]
+                        v = kwargs[axis.axisTag]
+                        if v is not None:
+                            unnormalized_variations[axis.axisTag] = kwargs[axis.axisTag]
+                    if generic in kwargs and axis.axisTag not in variations:
+                        unnormalized_variations[axis.axisTag] = kwargs[generic]
 
             self.addVariations(unnormalized_variations)
     
@@ -337,23 +245,18 @@ class Style():
         if not self.tag == other.tag:
             return False
         if not self.font == other.font:
-            #print("different font")
             return False
         elif not self.fontSize == other.fontSize:
-            #print("different fontSize")
             return False
         
         for key, value in self.variations.items():
             if self.variations[key] != other.variations[key]:
-                #print("different variation")
                 return False
             
         if self.rotate != other.rotate:
-            #print("different rotate")
             return False
         
         if self.fill != other.fill:
-            #print("different fill")
             return False
         
         return True
@@ -368,6 +271,53 @@ class Style():
         keyed.update(kwargs)
         return Style(**keyed)
     
+    def complete_metrics(self):
+        c = False
+        a = False
+        d = False
+
+        if self.capHeight is None and "c" in self.metrics:
+            c = True
+        elif self.ascender is None and "a" in self.metrics:
+            a = True
+
+        if self.descender is None and "d" in self.metrics:
+            d = True    
+
+        try:
+            if "OS/2" in self.font.font.ttFont:
+                os2 = self.font.font.ttFont["OS/2"]
+                if c:
+                    self.capHeight = os2.sCapHeight if hasattr(os2, "sCapHeight") else 0
+                    if self.capHeight == 0:
+                        self.capHeight = os2.sTypoAscender if hasattr(os2, "sTypoAscender") else 750
+                if a:
+                    self.ascender = os2.sTypoAscender if hasattr(os2, "sTypoAscender") else 0
+                    if self.ascender == 0:
+                        self.ascender = os2.sCapHeight if hasattr(os2, "sCapHeight") else 750
+                if d:
+                    self.descender = -os2.sTypoDescender if hasattr(os2, "sTypoDescender") else 250
+            
+            # TODO does this every happen?
+            elif hasattr(self.font.font, "info"):
+                if c:
+                    self.capHeight = self.font.font.info.capHeight
+                if a:
+                    self.ascender = self.font.font.info.ascender
+                if d:
+                    self.descender = -self.font.font.info.descender
+            
+            # TODO also does this ever happen?
+            elif hasattr(self.font.font, "defaultInfo"):
+                if c:
+                    self.capHeight = self.font.font.defaultInfo.capHeight
+                if a:
+                    self.ascender = self.font.font.defaultInfo.ascender
+                if d:
+                    self.descender = -self.font.font.defaultInfo.descender
+        except AttributeError:
+            pass
+    
     def addVariations(self, variations, limits=dict()):
         for k, v in self.normalizeVariations(variations).items():
             self.variations[k] = v
@@ -376,13 +326,14 @@ class Style():
     
     def normalizeVariations(self, variations):
         scale = self.scaleVariations
+        roll = self.rollVariations
+
         for k, v in variations.items():
             try:
                 axis = self.axes[k]
             except KeyError:
-                #print("Invalid axis", self.fontFile, k)
                 continue
-                #raise Exception("Invalid axis", self.fontFile, k)
+
             if v == "min":
                 variations[k] = axis.minValue
             elif v == "max":
@@ -390,14 +341,19 @@ class Style():
             elif v == "default":
                 variations[k] = axis.defaultValue
             elif scale:
-                _v = max(0, min(1, v))
+                vv = v
+                if roll:
+                    vv = v%2
+                    if 1 < vv < 2:
+                        vv = 2 - vv
+                _v = max(0, min(1, vv))
                 variations[k] = float(abs(axis.maxValue-axis.minValue)*_v + axis.minValue)
             else:
                 if v < axis.minValue or v > axis.maxValue:
                     variations[k] = max(axis.minValue, min(axis.maxValue, v))
                     print("----------------------")
                     print("Invalid Font Variation")
-                    print(self.fontFile, self.axes[k].axisTag, v)
+                    print(self.font.path, self.axes[k].axisTag, v)
                     print("> setting", v, "<to>", variations[k])
                     print("----------------------")
                 else:
@@ -410,7 +366,7 @@ class Style():
         def stretcher(w, xp, i, p):
             np = (p.flatten(flatten) if flatten else p).nonlinear_transform(lambda x,y: (x if x < xp else x + w, y))
             if debug:
-                (np.record(_PenClass()
+                (np.record(P()
                     .line([(xp, -250), (xp, 1000)])
                     .outline()))
             return np
@@ -429,10 +385,10 @@ class Style():
             np = (p.flatten(flatten) if flatten else p).nonlinear_transform(lambda x,y: (x if is_left((xdsc, -250), (xasc, 1000), (x, y)) else x + w, y))
             if debug:
                 (np
-                    .record(_PenClass()
+                    .record(P()
                         .line([(xdsc, -250), (xasc, 1000)])
                         .outline())
-                    .record(_PenClass()
+                    .record(P()
                         .moveTo((x0+50/2, y0+50/2))
                         .dots(radius=50)))
             return np
@@ -456,7 +412,7 @@ class Style():
             elif align == "mxy":
                 np.translate(0, -h)
             if debug:
-                (np.record(_PenClass()
+                (np.record(P()
                     .line([(0, yp), (p.ambit().point("E").x, yp)])
                     .outline()))
             return np
@@ -477,10 +433,10 @@ class Style():
             np = (p.flatten(flatten) if flatten else p).nonlinear_transform(lambda x,y: (x, y if not is_left(p0, p1, (x, y)) else y + h))
             if debug:
                 (np
-                    .record(_PenClass()
+                    .record(P()
                         .line([p0, p1])
                         .outline())
-                    .record(_PenClass()
+                    .record(P()
                         .moveTo((x0+50/2, y0+50/2))
                         .dots(radius=50)))
             return np
@@ -524,8 +480,11 @@ class StyledString(FittableMixin):
         #self.glyphs = self.style.font.font.getGlyphRun(self.text, features=self.features, varLocation=self.variations)
         x = 0
         for glyph in self.glyphs:
-            glyph.frame = Rect(x+glyph.dx, glyph.dy, glyph.ax, self.style.capHeight)
+            glyph.frame = Rect(x+glyph.dx, glyph.dy, glyph.ax, self.style._asc)
+            if "d" in self.style.metrics:
+                glyph.frame = glyph.frame.expand(self.style.descender, "N")
             x += glyph.ax
+        
         self.getGlyphFrames()
     
     def trackFrames(self, space_width=0):
@@ -582,7 +541,6 @@ class StyledString(FittableMixin):
             self.trackFrames(space_width=space_width)
 
         for glyph in self.glyphs:
-            #print(glyph, glyph.frame)
             glyph.frame = glyph.frame.scale(self.scale())
 
         if self.style.trackingMode == 0:
@@ -600,7 +558,10 @@ class StyledString(FittableMixin):
         return self.getGlyphFrames()[-1].frame.point("SE").x
     
     def height(self):
-        return self.style.capHeight*self.scale()
+        asc = self.style._asc * self.scale()
+        if "d" in self.style.metrics:
+            asc += self.style.descender * self.scale()
+        return asc
     
     def textContent(self):
         return self.text
@@ -707,7 +668,7 @@ class StyledString(FittableMixin):
         self.resetGlyphRun()
         return adjusted
     
-    def scalePenToStyle(self, glyph, in_pen):
+    def scalePenToStyle(self, glyph, in_pen, idx):
         s = self.scale()
         t = Transform()
         try:
@@ -725,12 +686,15 @@ class StyledString(FittableMixin):
                     t = t.translate(0, bs)
                 except:
                     pass
+        
         t = t.scale(s)
-        t = t.translate(glyph.frame.x/self.scale(), glyph.frame.y/self.scale())
-        #t = t.translate(0, bs)
-        out_pen = _PenClass()
+        s = self.scale()
+        if s > 0:
+            t = t.translate(glyph.frame.x/s, glyph.frame.y/s)
+
+        out_pen = P()
         tp = TransformPen(out_pen, (t[0], t[1], t[2], t[3], t[4], t[5]))
-        ip = _PenClass().record(in_pen)
+        ip = P().record(in_pen)
         if self.style.mods and glyph.name in self.style.mods:
             w, mod = self.style.mods[glyph.name]
             mod(-1, ip)
@@ -739,12 +703,12 @@ class StyledString(FittableMixin):
             out_pen.rotate(self.style.rotate)
         
         # TODO this shouldn't be necessary
-        if True:
-            valid_values = []
-            for (move, pts) in out_pen.value:
-                if move != "addComponent":
-                    valid_values.append((move, pts))
-            out_pen.value = valid_values
+        # if True:
+        #     valid_values = []
+        #     for (move, pts) in out_pen.value:
+        #         if move != "addComponent":
+        #             valid_values.append((move, pts))
+        #     out_pen.value = valid_values
 
         return out_pen
     
@@ -752,64 +716,222 @@ class StyledString(FittableMixin):
         #attrs = dict(fill=self.style.fill)
         #if self.style.stroke:
         #    attrs["stroke"] = dict(color=self.style.stroke, weight=self.style.strokeWidth)
-        dp = _PenClass().f(self.style.fill)
+        dp = P().f(self.style.fill)
         if self.style.stroke:
             dp.s(self.style.stroke).sw(self.style.strokeWidth)
         return dp
-
-    def pens(self, frame=True) -> DraftingPens:
-        """
-        Vectorize text into a ``DATPens``, such that each glyph (or ligature) is represented by a single `DATPen` (or a ``DATPens`` in the case of a color font, which will then nest a `DATPen` for each layer of that color glyph)
-        """
-        self.resetGlyphRun()
-        self.style.font.font.addGlyphDrawings(self.glyphs, colorLayers=True)
+    
+    def buildLayeredGlyph(self, idx, glyph, output, layer, frame):
+        layerGlyph = P().record(layer)
+        if layerGlyph.v.value:
+            output.append(layerGlyph)
         
-        pens = _PensClass()
+        layerGlyph.data(glyphName=f"{glyph.name}_layer_{idx}")
+
+        #print(">>>>>>>>>>>>", layer.method)
+
+        if layer.method == "drawPathSolid":
+            layerGlyph.f(layer.data["color"])
+        else:
+            gradientGlyph = P()
+            if layer.method == "drawPathLinearGradient":
+                (gradientGlyph
+                    .line([layer.data["pt1"], layer.data["pt2"]])
+                    .fssw(-1, 0, 2)
+                    #.translate(frame.x, 0)
+                    )
+            elif layer.method == "drawPathSweepGradient":
+                gradientGlyph.moveTo(layer.data["center"])
+            elif layer.method == "drawPathRadialGradient":
+                gradientGlyph.line([layer.data["startCenter"], layer.data["endCenter"]])
+            else:
+                print(">", layer.method)
+                gradientGlyph.rect(frame)
+            
+            (layerGlyph
+                .f(-1)
+                .attr(COLR=[layer.method, layer.data])
+                .data(substructure=gradientGlyph))
+    
+    def addBRGlyphDrawings(self, glyphs):
+        ax = 0
+        surface = BRPathCollectorSurface()
+
+        self.style.font._brFont.setLocation(self.variations)
+
+        if self.style.palette:
+            p = self.style.palette
+            if isinstance(p, int):
+                palette = self.style.font._brFont.getPalette(p)
+            else:
+                palette = p
+        else:
+            palette = None
+
+        with surface.canvas((0, 0, 1000, 1000)) as canvas:
+            for glyph in glyphs:
+                frame = Rect(
+                    ax + glyph.dx,
+                    glyph.dy,
+                    glyph.ax,
+                    self.style._asc) # how does ay play in?
+                
+                output = P().data(glyphName=glyph.name, frame=frame)
+
+                with canvas.savedState():
+                    #canvas.translate(glyph.dx, glyph.dy)
+
+                    self.style.font._brFont.drawGlyph(glyph.name, canvas, palette=palette)
+
+                    layers = canvas.paths
+
+                    if isinstance(layers, BRPathCollectorRecordingPen):
+                        if layers.method == "drawPathSolid": # trad font
+                            output.record(layers).f(layers.data["color"])
+                            layers = None
+                        else:
+                            layers = [layers]
+                    
+                    if layers:
+                        for idx, layer in enumerate(layers):
+                            self.buildLayeredGlyph(idx, glyph, output, layer, frame)
+                    
+                    canvas.paths = []
+
+                #canvas.translate(glyph.ax, glyph.ay)
+
+                glyph.glyphDrawing = output
+                #ax += glyph.ax
+
+    def pens(self) -> P:
+        """
+        Vectorize text into a `P`, such that each glyph (or ligature) is represented by a single `P` (or a `P` in the case of a color font, which will then nest a `P` for each layer of that color glyph)
+        """
+
+        self.resetGlyphRun()
+
+        colrv1 = self.style.font._colrv1
+        brFont = self.style.font._brFont
+
+        if brFont:
+            self.addBRGlyphDrawings(self.glyphs)
+
+        elif not self.style.no_shapes:
+            self.style.font.font.addGlyphDrawings(self.glyphs, colorLayers=True)
+        
+        pens = P()
         for idx, g in enumerate(self.glyphs):
+
+            # TODO this is sketchy but seems to correct
+            # some line-spacing issues with arabic?
+            norm_frame = g.frame
+            norm_frame = Rect(g.frame.x, 0, g.frame.w, self.style._asc*self.scale())
+
             dp_atom = self._emptyPenWithAttrs()
-            if len(g.glyphDrawing.layers) == 1:
-                dp_atom.value = self.scalePenToStyle(g, g.glyphDrawing.layers[0][0]).value
-                dp_atom.typographic = True
-                dp_atom.addFrame(g.frame)
-                dp_atom.glyphName = g.name
+            if self.style.no_shapes:
+                if callable(self.style.show_frames):
+                    dp_atom.record(P().rect(self.style.show_frames(g.frame)).outline(4))
+                else:
+                    dp_atom.record(P().rect(g.frame).outline(1 if self.style.show_frames is True else self.style.show_frames))
+                dp_atom.data(
+                    frame=norm_frame,
+                    glyphName=g.name
+                )
+                # dp_atom.typographic = True
+                # dp_atom.addFrame(norm_frame)
+                # dp_atom.glyphName = g.name
+            
+            elif not brFont and len(g.glyphDrawing.layers) == 1:
+                dp_atom.v.value = self.scalePenToStyle(g, g.glyphDrawing.layers[0][0], idx).v.value
+                
+                if "d" in self.style.metrics:
+                    dp_atom.translate(0, self.style.descender*self.scale())
+                
+                dp_atom.data(
+                    frame=norm_frame,
+                    glyphName=g.name
+                )
+
+                if self.style.show_frames:
+                    if callable(self.style.show_frames):
+                        #dp_atom.record(P().rect(self.style.show_frames(g.frame)).outline(4))
+                        dp_atom.rect(self.style.show_frames(g.frame))
+                    else:
+                        dp_atom.record(P().rect(g.frame).outline(1 if self.style.show_frames is True else self.style.show_frames))
+                        #dp_atom.rect(g.frame)
                 if self.style.q2c:
                     dp_atom.q2c()
                 if self.style.removeOverlap:
                     dp_atom.removeOverlap()
+
+            elif brFont:
+                dp_atom = g.glyphDrawing
+                dp_atom.layered = len(dp_atom) > 1 or colrv1
+
+                for idx, layer in enumerate(dp_atom):
+                    layer.v.value = self.scalePenToStyle(g, layer, idx).v.value
+
+                    ss = layer.data("substructure")
+                    if ss:
+                        ss.v.value = self.scalePenToStyle(g, ss, idx).v.value
+                
+                if not dp_atom.layered:
+                    dp_atom = dp_atom[0]
+                
+                dp_atom.data(
+                   frame=norm_frame,
+                   glyphName=g.name)
+
+                if self.style.q2c:
+                    dp_atom.q2c()
+                if self.style.removeOverlap:
+                    dp_atom.removeOverlap()
+                
             else:
-                dp_atom = _PensClass()
+                dp_atom = P()
                 dp_atom.layered = True
                 for lidx, layer in enumerate(g.glyphDrawing.layers):
                     dp_layer = self._emptyPenWithAttrs()
                     #dp_layer.value = layer[0].value
-                    dp_layer.value = self.scalePenToStyle(g, layer[0]).value
+                    dp_layer.v.value = self.scalePenToStyle(g, layer[0], idx).v.value
                     if isinstance(self.style.palette, int):
                         dp_layer.f(self.style.font.font.colorPalettes[self.style.palette][layer[1]])
                     else:
                         dp_layer.f(self.style.palette[layer[1]])
-                    if len(dp_layer.value) > 0:
+                    if len(dp_layer.v.value) > 0:
                         #dp_layer.addFrame(g.frame, typographic=True)
-                        dp_layer.glyphName = f"{g.name}_layer_{lidx}"
+                        dp_layer.data(glyphName=f"{g.name}_layer_{lidx}")
+                        #dp_layer.glyphName = 
                         dp_atom += dp_layer
-                dp_atom.addFrame(g.frame, typographic=True)
-                dp_atom.glyphName = g.name
+                
+                dp_atom.data(
+                    frame=norm_frame,
+                    glyphName=g.name
+                )
+                # dp_atom.addFrame(norm_frame, typographic=True)
+                # dp_atom.glyphName = g.name
             
             #dp_atom._parent = pens
-            pens.append(dp_atom, allow_blank=self.style.include_blanks)
+            if self.style.meta:
+                dp_atom.data(**self.style.meta)
+            
+            pens.append(dp_atom)
 
         if self.style.reverse:
             pens.reversePens()
         
-        for k, v in self.style.data.items():
-            pens.data[k] = v
+        pens.data(**self.style.data)
 
-        return pens
+        ro = pens
+        if self.style._stst:
+            ro._stst = self
+        return ro
 
-    def pen(self, frame=True) -> DraftingPen:
+    def pen(self, frame=True) -> P:
         """
-        Vectorize all text into single ``DATPen``
+        Vectorize all text into single `P`
         """
-        return self.pens(frame=frame).pen()
+        return self.pens().pen()
 
 class SegmentedString(FittableMixin):
     def __init__(self, text, styles):
@@ -842,15 +964,19 @@ class SegmentedString(FittableMixin):
             adjusted = s.shrink() or adjusted
         return adjusted
 
-    def pens(self):
-        pens = _PensClass()
+    def pens(self, flat=True):
+        pens = P()
         x_off = 0
         for s in self.strings:
-            dps = s.pens(frame=True)
-            if dps.layered:
-                pens.layered = True
+            dps = s.pens()
+            #if dps.layered:
+            #    pens.layered = True
             dps.translate(x_off, 0)
-            pens.extend(dps._pens)
+            if flat:
+                pens.extend(dps._els)
+            else:
+                if s.style.lang:
+                    dps.data(lang=s.style.lang)
+                pens.append(dps)
             x_off += dps.ambit().w
-        return pens
-        #return DATPens([s.pens(frame=True) for s in self.strings])
\ No newline at end of file
+        return pens
\ No newline at end of file
